#include <stdio.h>
#include <conio.h>
#include "AliIpl.h"

static unsigned short cpu_type=0;
//idle state: scl in, sda in, scl=1, sda=1
static unsigned iodir_addr, ioin_addr, ioout_addr, iodir_val, ioout_val, scl_mask, sda_mask;

typedef struct
{
	unsigned in, out, dir;
} GPIO;

static GPIO gpio[] = { 	{ 0xB8000050, 0xB8000054, 0xB8000058}, 
			{ 0xB80000D0, 0xB80000D4, 0xB80000D8},
			{ 0xB80000E4, 0xB80000E8, 0xB80000EC},
			{ 0xB80000F0, 0xB80000F4, 0xB80000F8} };

static bool SetDir()
{
	if (!IplWriteMem(&iodir_val, iodir_addr, 4))
	{
		puts("Can't write IODIR !");
		return false;
	}
	return true;
}

static bool SetOut()
{
	if (!IplWriteMem(&ioout_val, ioout_addr, 4))
	{
		puts("Can't write IOOUT !");
		return false;
	}
	return true;
}

static unsigned GetIn()
{
	unsigned u;
	if (!IplReadMem(&u, ioin_addr, 4))
	{
		puts("Can't read IOIN !");
		return 0;
	}
	return u;
}
			
static bool Attach(int sda_pin, int scl_pin)
{
	if ((sda_pin>>5)!=(scl_pin>>5))
	{
		puts("SCL and SDA are in different GPIO blocks! Unsupported!");
		IplClose();
		return false;
	}
	iodir_addr = gpio[sda_pin>>5].dir;
	ioin_addr = gpio[sda_pin>>5].in;
	ioout_addr = gpio[sda_pin>>5].out;
	sda_mask = 1<<(sda_pin & 0x1F);
	scl_mask = 1<<(scl_pin & 0x1F);
	if (!IplReadMem(&iodir_val, iodir_addr, 4))
	{
		puts("Can't read IODIR !");
		IplClose();
		return false;
	}
	iodir_val &= (~sda_mask) & (~scl_mask);
	if (!IplReadMem(&ioout_val, ioout_addr, 4))
	{
		puts("Can't read IOOUT !");
		IplClose();
		return false;
	}
	ioout_val |= sda_mask | scl_mask;
	if (!SetDir())
	{
		IplClose();
		return false;
	}
	if (!SetOut())
	{
		IplClose();
		return false;
	}
	return true;
}

bool I2COpen()
{
  	if (!IplOpen())
        {
          	puts("Port open failed !");
          	return false;
        }
        if (!IplStart())
        {
          	puts("Can't handshake with BootROM !");
          	IplClose();
          	return false;
        }
	if (!IplReadMem(&cpu_type, 0xB8000002, 2))
	{
          	puts("Can't read CPU revision !");
          	IplClose();
          	return false;
        }
	switch(cpu_type)
	{
		case 0x3602:
		{
			puts("Found ALi M3602");
			return Attach(64, 65);
		}
		case 0x3603:
		{
			puts("Found ALi M3606");

			unsigned u;
			IplReadMem(&u, 0xB80000F8, 4);
			u |= 0x1000;
			IplWriteMem(&u, 0xB80000F8, 4);

			IplReadMem(&u, 0xB80000F4, 4);
			u &= ~0x1000;
			IplWriteMem(&u, 0xB80000F4, 4);

			IplReadMem(&u, 0xB8000088, 4);
			u |= 0x3000000;
			IplWriteMem(&u, 0xB8000088, 4);

			return Attach(43, 44);
		}
		default:
			puts("Unknown CPU type !");
			IplClose();
			return false;
	}
}

void I2CClose()
{
	IplClose();
}

//issue I2C start
//preconditions:  SDA=1, SCL=1
//postconditions: SDA=out0, SCL=out0
static void start()
{
	iodir_val |= sda_mask | scl_mask;
	SetDir();			
	ioout_val &= ~sda_mask;
	SetOut();
	ioout_val &= ~scl_mask;
	SetOut();
}

static void stop()
{
	iodir_val |= sda_mask | scl_mask;
	SetDir();			
	ioout_val &= ~sda_mask;
	SetOut();
	ioout_val |= scl_mask;
	SetOut();
	ioout_val |= sda_mask;
	SetOut();
}

static void set_bit(int val)
{
	iodir_val |= sda_mask | scl_mask;
	SetDir();			
	ioout_val &= ~scl_mask;
	SetOut();
	if (val)
	{
		ioout_val |=sda_mask;
	}
	else
	{
		ioout_val &= ~sda_mask;
	}
	SetOut();
	ioout_val |= scl_mask;
	SetOut();
	ioout_val &= ~scl_mask;
	SetOut();
}

static int get_bit()
{
	iodir_val &= ~sda_mask;
	SetDir();			
	ioout_val |=sda_mask;
	SetOut();
	ioout_val |=scl_mask;
	SetOut();
	int val = (GetIn() & sda_mask) ? 1 : 0;
	ioout_val &= ~scl_mask;
	SetOut();			
	return val;	
}

static int put_byte(unsigned char val)
{
	for(int i=0; i<8; i++)
	{
		set_bit(val>>7);
		val<<=1;
	}
	return get_bit();
}

static unsigned char get_byte(int ack)
{
	unsigned char val=0;
	for(int i=0; i<8; i++)
	{
		val<<=1;
		val |= get_bit();
	}
	set_bit(ack);				
	return val;
}

bool I2CWrite(unsigned char saddr, const void *buf, int size)
{
	start();
	if (put_byte(saddr & 0xFE))
	{
		puts("NAK !");
		stop();
		return false;
	}
	const unsigned char *ptr = (const unsigned char *)buf;
	for(int i=0; i<size; i++)
	{
		put_byte(ptr[i]);
	}
	stop();
	return true;
}

bool I2CRead(unsigned char saddr, void *buf, int size)
{
	start();
	if (put_byte(saddr | 1))
	{
		puts("NAK !");
		stop();
		return false;
	}
	unsigned char *ptr = (unsigned char *)buf;
	for(int i=0; i<size; i++)
	{
		ptr[i] = get_byte((i==(size-1)) ? 1 : 0);
	}
	stop();
	return true;
}

void I2CTest()
{
	puts("SDA=1,i\tSCL=1,i");
	getch();
	iodir_val |= sda_mask;
	SetDir();
	puts("SDA=1,o\tSCL=1,i");
		
	getch();
	iodir_val |= scl_mask;
	SetDir();
	puts("SDA=1,o\tSCL=1,o");

	getch();
	ioout_val &= ~sda_mask;
	SetOut();
	puts("SDA=0,o\tSCL=1,o");

	getch();
	ioout_val &= ~scl_mask;
	SetOut();
	puts("SDA=0,o\tSCL=0,o");
}
