#include <stdio.h>
#include <string.h>
#include <comm/comm.h>
#include <util/conutil.h>
#include "AliIpl.h"
#include "AliIplCmd.h"

static unsigned IplLastError=0;

unsigned IplGetLastError(void)
{
  return IplLastError;
}

void IplSetLastError(unsigned error)
{
  IplLastError = error;
}

bool IplOpen(char *port)
{
  	if (port==0)
	{
  		if (!InitComm("VCP0"))
  			if (!InitComm("Serial0"))
  				return false;
	}
  	else
	{
  		if (!InitComm(port))
  			return false;
	}
	SetFormat(8, 'E', 2);
	SetBaudrate(115200);
	return true;	
}

void IplClose()
{
  CloseComm();
}

bool IplPing()
{
  unsigned char cmd = CMD_PING;
  WriteComm(&cmd, 1);
  if (ReadComm(&cmd, 1)!=1)
  {
    IplSetLastError(IPL_ECOMM);
    return false;
  }
  if (cmd!=CMD_PING)
  {
    printf("%c", cmd);
    IplSetLastError(IPL_ERESP);
    return false;
  }
  return true;
}

bool IplCall(unsigned addr)
{
  unsigned char cmd[5];
  cmd[0] = CMD_CALL;
  *(unsigned *)(cmd+1) = addr;
  WriteComm(cmd, 5);
  return true;
}

static unsigned char log2(unsigned short len)
{
  unsigned char ll;
  for(ll=0; ll<16; ll++)
     if ((1<<ll)==len)
     	break;
  return ll;	
}

bool IplReadMem(void *buf, unsigned addr, unsigned short len)
{
  unsigned char ll = log2(len);
  if (ll>15)
  {
  	IplSetLastError(IPL_EPARAM);
  	return false;
  }
  unsigned char cmd[5];
  cmd[0] = CMD_READ | ll;
  *(unsigned *)(cmd+1) = addr;
  WriteComm(&cmd, 5);
  if (ReadComm(buf, len)!=len)
  {
    IplSetLastError(IPL_ECOMM);
    return false;
  }
  return true;
}

bool IplWriteMem(void *buf, unsigned addr, unsigned short len)
{
  unsigned char ll = log2(len);
  if (ll>15)
  {
  	IplSetLastError(IPL_EPARAM);
  	return false;
  }
  unsigned char cmd[5];
  cmd[0] = CMD_WRITE | ll;
  *(unsigned *)(cmd+1) = addr;
  WriteComm(&cmd, 5);
  WriteComm(buf, len);
  return true;
}

unsigned char IplGetResponse()
{
  unsigned char cmd;
  if (ReadComm(&cmd, 1)!=1)
  {
    IplSetLastError(IPL_ECOMM);
    return 0;
  }
  return cmd;
}

bool IplStart()
{
	puts("Waiting for reset...");
	SetReadTimeout(1, 15);
	unsigned char c=0;
//	while(c!='A')
//		ReadComm(&c, 1);
	while(!IplPing())
		Fan();
	puts("\rGot ping !");
	unsigned char cmd = 0;
	for(int i=0; i<8; i++)
		IplWriteMem(&cmd, 0xB8000000, 1);
	FlushComm();
	while(ReadComm(&cmd, 1)==1);
	SetReadTimeout(5, 1000);
	return true;
}
