#include <stdio.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "AliIpl.h"
#include "util/conutil.h"

#ifdef COMPRESS
#include "compress/FileCompressor/FileCompressor.h"
#endif

#define MEMBLOCKSIZE   32768

bool IplReadMemToFile(unsigned addr, unsigned len, const char *file)
{
  unsigned hf;
  unsigned char buf[MEMBLOCKSIZE];

#ifdef COMPRESS
  FileCompressor fc(0x4000);
  if (!fc.create(file))
#else
  hf = open(file, O_CREAT | O_TRUNC | O_RDWR | O_BINARY, S_IWRITE);
  if (hf==0xFFFFFFFF)
#endif
  {
    printf("File %s create error !\n",file);
    IplSetLastError(IPL_EFILE);
    return false;
  }
  unsigned len_org = len;
  while(len>MEMBLOCKSIZE)
  {
    if (!IplReadMem(buf, addr, MEMBLOCKSIZE))
    {
       printf("Address %X read failed, code %X !\n", addr, IplGetLastError());
    }
    else
    {
#ifdef COMPRESS
      fc.write(buf, MEMBLOCKSIZE);
#else
      write(hf, buf, MEMBLOCKSIZE);
#endif
    }
    ProgressBar(len_org-len, len_org);
    addr+=MEMBLOCKSIZE;
    len-=MEMBLOCKSIZE;
  }
  if (!IplReadMem(buf, addr, len))
  {
    printf("Address %X read failed, code %X !\n", addr, IplGetLastError());
  }
  else
  {
#ifdef COMPRESS
      fc.write(buf, len);
#else
      write(hf, buf, len);
#endif
  }
  ProgressBar(1, 1);
  puts("");
#ifdef COMPRESS
       fc.close();
#else
       close(hf);
#endif
  return true;
}

bool IplWriteMemFromFile(unsigned addr, const char *file)
{
  unsigned hf;
  unsigned char buf[MEMBLOCKSIZE];

  hf = open(file, O_RDONLY | O_BINARY);
  if (hf==0xFFFFFFFF)
  {
    printf("File %s open error !\n",file);
    IplSetLastError(IPL_EFILE);
    return false;
  }
  unsigned len = filelength(hf);
  unsigned len_org = len;
  while(len>MEMBLOCKSIZE)
  {
    ProgressBar(len_org-len, len_org);
    read(hf, buf, MEMBLOCKSIZE);
    if (!IplWriteMem(buf, addr, MEMBLOCKSIZE))
    {
       printf("Address %X write failed, code %X !\n", addr, IplGetLastError());
    }
    addr+=MEMBLOCKSIZE;
    len-=MEMBLOCKSIZE;
  }
  read(hf, buf, len);
  if (!IplWriteMem(buf, addr, len))
  {
    printf("Address %X write failed, code %X !\n", addr, IplGetLastError());
  }
  ProgressBar(1, 1);
  puts("");
  close(hf);
  return true;
}
