#include <stdio.h>
#include <memory.h>
#include <util/ConUtil.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "AliI2C.h"
#include "crc16ibm.h"

unsigned SwapD(unsigned val)
{
	return (val>>24) | ((val>>8) & 0x0000FF00) | ((val<<8) & 0x00FF0000) | (val<<24);
}

void main(int argc, char *argv[])
{
	if (argc!=3)
	{
		printf("Usage: %s <infile.abs> <outfile>\n", argv[0]);
		return;
	}
	int hfi = open(argv[1], O_RDONLY | O_BINARY);
	if (hfi<0)
	{
		puts("Can't open input file !");
		return;
	}
	lseek(hfi, 8, SEEK_SET);
	unsigned ofs;
	read(hfi, &ofs, 4);
	ofs = SwapD(ofs);
	printf("maincode is at %X\n", ofs);

	lseek(hfi, ofs+4, SEEK_SET);
	unsigned size;
	read(hfi, &size, 4);
	size = SwapD(size)-0x70; //w/o hdr size
	printf("maincode body size is %X\n", size);

	lseek(hfi, ofs+0x68, SEEK_SET);
	unsigned char cmd[26];
	read(hfi, cmd, 8); //salt

	lseek(hfi, ofs+0x80, SEEK_SET); //code

	int hfo = open(argv[2], O_CREAT | O_TRUNC | O_RDWR | O_BINARY, S_IWRITE);
	if (hfo<0)
	{
		puts("Can't create output file !");
		close(hfi);
		return;
	}

  	setbuf(stdout, 0);
        if (!I2COpen())
        {
          	puts("Bus open failed !");
          	close(hfo);
          	close(hfi);
          	return;
        }
        puts("Decrypting...");
	for(int i=0; i<1024; i+=16)
	{
		ProgressBar(i, 1024);
		read(hfi, cmd+8, 16);
		unsigned short crc = crc16ibm(cmd, 24);
		cmd[24] = crc>>8;
		cmd[25] = crc;
		if (!I2CWrite(0xC4, cmd, 26))
        	{
        		puts("I2C Write failed !");
        		close(hfo);
        		close(hfi);
        		I2CClose();
        		return;
        	}
        	unsigned char rsp[26];
        	if (!I2CRead(0xC4, rsp, 26))
        	{
        		puts("I2C Read failed !");
        		close(hfo);
        		close(hfi);
        		I2CClose();
        		return;
        	}
        	write(hfo, rsp+8, 16);
	}
        I2CClose();
        size-=1024; //w/o crypted 1k
        unsigned char *buf = new unsigned char[size];
        read(hfi, buf, size);
        write(hfo, buf, size);
	ProgressBar(1, 1);
        delete buf;
        close(hfo);
        close(hfi);
        puts("Done !");
}
